# Generated by Django 2.1.5 on 2019-04-10 07:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pynotify', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='notification',
            name='extra_data',
            field=models.TextField(blank=True, null=True, verbose_name='extra data'),
        ),
    ]
