# Generated by Django 3.0.2 on 2020-07-23 13:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pynotify', '0004_add_admin_template'),
    ]

    operations = [
        migrations.AddField(
            model_name='adminnotificationtemplate',
            name='is_active',
            field=models.BooleanField(default=True, verbose_name='is active'),
        ),
    ]
