# Generated by Django 3.0.2 on 2020-01-17 17:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pynotify', '0002_notification_extra_data'),
    ]

    operations = [
        migrations.AlterField(
            model_name='notificationrelatedobject',
            name='name',
            field=models.CharField(blank=True, max_length=200, null=True, verbose_name='name'),
        ),
    ]
