# Generated by Django 2.2.15 on 2020-12-07 08:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pynotify', '0005_adminnotificationtemplate_is_active'),
    ]

    operations = [
        migrations.AddField(
            model_name='adminnotificationtemplate',
            name='send_push',
            field=models.BooleanField(default=False, verbose_name='send push notification'),
        ),
    ]
